/*
  highlycomposites.cc

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#include <iostream>
#include <string>
#include <libgen.h>
#include <gmpxx.h>

mpz_class n_divisors(const mpz_class n)
{
  mpz_class d(1), sqrt_n, c(0);

  mpz_sqrt(sqrt_n.get_mpz_t(), n.get_mpz_t());
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      ++c;
      if (d != n/d) ++c;
    }
  return c;
}

void mpz_nextcomposite(mpz_t rop, const mpz_t op)
{
  mpz_add_ui(rop, op, 1);
  for (; ; mpz_add_ui(rop, rop, 1))
    if (!mpz_probab_prime_p(rop, 25))
      break;
}

int usage(std::ostream &os, int argc, char *argv[])
{
  (void)argc;
  os << "Usage:\n\t" << basename(argv[0]) << " [-u upper]\n";
  return os == std::cerr ? 1 : 0;
}
int main(int argc, char *argv[])
{
  int a;
  mpz_class n, u(1000000), m(0), c;

  for (a=1; a<argc; ++a)
    if (std::string(argv[a]) == "-h")
      return usage(std::cerr, argc, argv);
    else if (std::string(argv[a]) == "-u" && a+1 < argc)
      u = argv[++a];
    else
      break;
  
  for (n = 1; n < 3 && n < u; ++n)
    if ((c=n_divisors(n)) > m) {
      std::cout << n << std::endl;
      m = c;
    }
  for (; n < u; mpz_nextcomposite(n.get_mpz_t(), n.get_mpz_t()))
    if ((c=n_divisors(n)) > m) {
      std::cout << n << std::endl;
      m = c;
    }

  return 0;
}
