/*
  prime_polynomial_progression.cc

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#include <iostream>
#include <string>
#include <vector>
#include <libgen.h>
#include <gmpxx.h>

int usage(std::ostream &os, int argc, char *argv[])
{
  (void)argc;
  os << "Usage:\n\t" << basename(argv[0]) << " [-l lower] [-u upper] [-c coefficient ...] [-k number]\n";
  return os == std::cerr ? 1 : 0;
}
int main(int argc, char *argv[])
{
  int a;
  mpz_class n, l(1), u(1000000000), k(17), i;
  std::vector<std::vector<mpz_class> > c;

  for (a=1; a<argc; ++a)
    if (std::string(argv[a]) == "-h")
      return usage(std::cout, argc, argv);
    else if (std::string(argv[a]) == "-l" && a+1 < argc)
      l = argv[++a];
    else if (std::string(argv[a]) == "-u" && a+1 < argc)
      u = argv[++a];
    else if (std::string(argv[a]) == "-c" && ++a < argc) {
      c.push_back(std::vector<mpz_class>());
      for (; a<argc; ++a) {
	mpz_class n;
	try { n = mpz_class(argv[a]); }
	catch (...) { --a; break; }
	c[c.size()-1].push_back(n);
      }
    }
    else if (std::string(argv[a]) == "-k" && a+1 < argc)
      k = argv[++a];
    else
      break;

  for (; l<u; mpz_nextprime(l.get_mpz_t(), l.get_mpz_t())) {
    for (size_t t=0; t<c.size(); ++t) {
      c[t][0] = l;
      for (i=0; ; ++i) {
	n = 0;
	for (size_t j=0; j<c[t].size(); j++) {
	  mpz_class x;
	  mpz_pow_ui(x.get_mpz_t(), i.get_mpz_t(), j);
	  n += c[t][j]*x;
	}
	if (!mpz_probab_prime_p(n.get_mpz_t(), 25)) break;
      }
      if (i>k) {
	std::cout << i << "\t";
	for (size_t j=0; j<c[t].size(); j++) std::cout << c[t][j] << "\t";
	std::cout << std::endl;
      }
    }
  }
  return 0;
}
